// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/driver/device.c $
// $Rev: 54019 $
// $Date: 2023-12-12 09:15:32 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model		Vendor	Device	SubVen	SubDev	type

	{ "16AISS8AO4",	0x10B5, 0x9056, 0x10B5, 0x3172,	GSC_DEV_TYPE_16AISS8AO4	},
	{ NULL }
};



//*****************************************************************************
static void _burst_sync_compute(dev_data_t* dev)
{
	if ((dev->cache.gsc_acr_32 & 0xFFF) >= 0x400)
		dev->cache.burst_sync	= 1;
	else
		dev->cache.burst_sync	= 0;
}



//*****************************************************************************
static void _dmdma_compute(dev_data_t* dev)
{
	if ((dev->cache.gsc_acr_32 & 0xFFF) >= 0x0102)
		dev->cache.dmdma	= 1;
	else
		dev->cache.dmdma	= 0;
}



//*****************************************************************************
static void _fref_compute(dev_data_t* dev)
{
	switch (dev->cache.gsc_acr_32 & (D19 | D18))
	{
		default:
		case 0:			dev->cache.master_clock	= 40320000L;	break;
		case D18:		dev->cache.master_clock	= 40320000L;	break;
		case D19:		dev->cache.master_clock	= 40320000L;	break;
		case D19 | D18:	dev->cache.master_clock	= 40320000L;	break;
	}
}



//*****************************************************************************
static void _sys_io_cfg_compute(dev_data_t* dev)
{
	dev->cache.sys_io_cfg	= (dev->cache.gsc_acr_32 & D21)
							? AISS8AO4_SYS_IO_CFG_ALT_SMA
							: AISS8AO4_SYS_IO_CFG_STANDARD;
}



//*****************************************************************************
static void _autocal_compute(dev_data_t* dev)
{
	dev->cache.autocal_ms	= 5000;

	if (((dev->cache.gsc_acr_32 & 0xFFF) >= 0x202) &&
		((dev->cache.gsc_acr_32 & 0xFFF) <= 0x2FF))
	{
		dev->cache.autocal_ai	= 1;
	}
	else
	{
		dev->cache.autocal_ai	= 0;
	}
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma0;
	u32	dma1;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(AISS8AO4_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_acr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_ACR));
		dev->vaddr.gsc_bcr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_BCR));
		dev->vaddr.gsc_boor_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_BOOR));
		dev->vaddr.gsc_diopr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_DIOPR));
		dev->vaddr.gsc_ibdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_IBDR));
		dev->vaddr.gsc_ibsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_IBSR));
		dev->vaddr.gsc_ibtr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_IBTR));
		dev->vaddr.gsc_icr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_ICR));
		dev->vaddr.gsc_psr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_PSR));
		dev->vaddr.gsc_obdr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_OBDR));
		dev->vaddr.gsc_obsr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_OBSR));
		dev->vaddr.gsc_obtr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_OBTR));
		dev->vaddr.gsc_ragr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_RAGR));
		dev->vaddr.gsc_rbgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_RBGR));
		dev->vaddr.gsc_rcgr_32		= GSC_VADDR(dev, GSC_REG_OFFSET(AISS8AO4_GSC_RCGR));

		// Data cache initialization.
		dev->cache.gsc_acr_32		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_acr_32);

		dev->cache.ai_chan_qty		= (dev->cache.gsc_acr_32 & D16) ? 4 : 8;
		dev->cache.ai_chans_max		= 8;
		dev->cache.ao_chan_qty		= (dev->cache.gsc_acr_32 & D17) ? 0 : 4;
		dev->cache.ao_chans_max		= 4;
		dev->cache.fifo_size_rx		= _256K;
		dev->cache.fifo_size_tx		= _256K;
		dev->cache.initialize_ms	= 3;
		dev->cache.ndiv_max_ai		= 0xFFFFFF;
		dev->cache.ndiv_max_ao		= 0xFFFFFF;
		dev->cache.ndiv_min_ai		= 20;
		dev->cache.ndiv_min_ao		= 40;
		dev->cache.rate_gen_qty		= 3;
		dev->cache.reg_aux_user		= 4;

		_fref_compute(dev);
		_autocal_compute(dev);
		_sys_io_cfg_compute(dev);
		_dmdma_compute(dev);
		_burst_sync_compute(dev);

		dev->cache.fgen_max_ai		= dev->cache.master_clock / dev->cache.ndiv_min_ai;
		dev->cache.fgen_max_ao		= dev->cache.master_clock / dev->cache.ndiv_min_ao;
		dev->cache.fgen_min_ai		= dev->cache.master_clock / dev->cache.ndiv_max_ai;
		dev->cache.fgen_min_ao		= dev->cache.master_clock / dev->cache.ndiv_max_ao;
		dev->cache.fsamp_max_ai		= dev->cache.fgen_max_ai;
		dev->cache.fsamp_max_ao		= dev->cache.fgen_max_ao;
		dev->cache.fsamp_min_ai		= dev->cache.fgen_min_ai;
		dev->cache.fsamp_min_ao		= dev->cache.fgen_min_ao;

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev);
		if (ret)	break;

		dma0	= GSC_DMA_SEL_STATIC | GSC_DMA_CAP_BMDMA_READ;
		dma1	= GSC_DMA_SEL_STATIC | GSC_DMA_CAP_BMDMA_WRITE;

		if (dev->cache.dmdma)
		{
			dma0	|= GSC_DMA_CAP_DMDMA_READ;
			dma1	|= GSC_DMA_CAP_DMDMA_WRITE;
		}

		ret	= gsc_dma_create(dev, dma0, dma1);

		if (ret == 0)
			os_metrics(dev, dev->vaddr.gsc_acr_32);

		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);
		dev_io_destroy(dev);
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


