// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/driver/ioctl.c $
// $Rev: 54019 $
// $Date: 2023-12-12 09:15:32 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTOCAL_AI_START		D3
#define	AUTOCAL_START			D28		// BCR
#define	AUTOCAL_PASS			D29		// BCR
#define	INIT_START				D31		// BCR

#define	IRQ_AUTOCAL_DONE		D0		// PSR



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:							arg[0]	= AISS8AO4_IOCTL_QUERY_ERROR;	break;
		case AISS8AO4_QUERY_AUTOCAL_AI:		arg[0]	= dev->cache.autocal_ai;		break;
		case AISS8AO4_QUERY_AUTOCAL_MS:		arg[0]	= dev->cache.autocal_ms;		break;
		case AISS8AO4_QUERY_BURST_SYNC:		arg[0]	= dev->cache.burst_sync;		break;
		case AISS8AO4_QUERY_CHANNEL_AI_MAX:	arg[0]	= dev->cache.ai_chans_max;		break;
		case AISS8AO4_QUERY_CHANNEL_AI_QTY:	arg[0]	= dev->cache.ai_chan_qty;		break;
		case AISS8AO4_QUERY_CHANNEL_AO_MAX:	arg[0]	= dev->cache.ao_chans_max;		break;
		case AISS8AO4_QUERY_CHANNEL_AO_QTY:	arg[0]	= dev->cache.ao_chan_qty;		break;
		case AISS8AO4_QUERY_COUNT:			arg[0]	= AISS8AO4_IOCTL_QUERY_LAST;	break;
		case AISS8AO4_QUERY_DEVICE_TYPE:	arg[0]	= dev->board_type;				break;
		case AISS8AO4_QUERY_DMDMA:			arg[0]	= dev->cache.dmdma;				break;
		case AISS8AO4_QUERY_FGEN_MAX_AI:	arg[0]	= dev->cache.fgen_max_ai;		break;
		case AISS8AO4_QUERY_FGEN_MAX_AO:	arg[0]	= dev->cache.fgen_max_ao;		break;
		case AISS8AO4_QUERY_FGEN_MIN_AI:	arg[0]	= dev->cache.fgen_min_ai;		break;
		case AISS8AO4_QUERY_FGEN_MIN_AO:	arg[0]	= dev->cache.fgen_min_ao;		break;
		case AISS8AO4_QUERY_FIFO_SIZE_RX:	arg[0]	= dev->cache.fifo_size_rx;		break;
		case AISS8AO4_QUERY_FIFO_SIZE_TX:	arg[0]	= dev->cache.fifo_size_tx;		break;
		case AISS8AO4_QUERY_FSAMP_MAX_AI:	arg[0]	= dev->cache.fsamp_max_ai;		break;
		case AISS8AO4_QUERY_FSAMP_MAX_AO:	arg[0]	= dev->cache.fsamp_max_ao;		break;
		case AISS8AO4_QUERY_FSAMP_MIN_AI:	arg[0]	= dev->cache.fsamp_min_ai;		break;
		case AISS8AO4_QUERY_FSAMP_MIN_AO:	arg[0]	= dev->cache.fsamp_min_ao;		break;
		case AISS8AO4_QUERY_INIT_MS:		arg[0]	= dev->cache.initialize_ms;		break;
		case AISS8AO4_QUERY_MASTER_CLOCK:	arg[0]	= dev->cache.master_clock;		break;
		case AISS8AO4_QUERY_NDIV_MAX_AI:	arg[0]	= dev->cache.ndiv_max_ai;		break;
		case AISS8AO4_QUERY_NDIV_MAX_AO:	arg[0]	= dev->cache.ndiv_max_ao;		break;
		case AISS8AO4_QUERY_NDIV_MIN_AI:	arg[0]	= dev->cache.ndiv_min_ai;		break;
		case AISS8AO4_QUERY_NDIV_MIN_AO:	arg[0]	= dev->cache.ndiv_min_ao;		break;
		case AISS8AO4_QUERY_RATE_GEN_QTY:	arg[0]	= dev->cache.rate_gen_qty;		break;
		case AISS8AO4_QUERY_REG_AUX_USER:	arg[0]	= dev->cache.reg_aux_user;		break;
		case AISS8AO4_QUERY_SYS_IO_CFG:		arg[0]	= dev->cache.sys_io_cfg;		break;
	}

	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	int		i;
	long	ms	= dev->cache.initialize_ms + 5000;
	u32		reg;
	int		ret;

	// Manually wait for completion as there is no Init Done interrupt.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, INIT_START);
	ret	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, INIT_START, 0);

	if (ret)
	{
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				(long) ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (reg & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_start(dev_data_t* dev, void* arg)
{
	// Initiate autocalibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, AUTOCAL_START, AUTOCAL_START);
	return(0);
}



//*****************************************************************************
static int _autocal(dev_data_t* dev, void* arg)
{
	u32			mask;
	long		ms		= dev->cache.autocal_ms + 5000;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	VADDR_T		va		= dev->vaddr.gsc_bcr_32;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Autocal Done interrupt.
	mask	= IRQ_AUTOCAL_DONE;
	value	= IRQ_AUTOCAL_DONE;
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_psr_32, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= AISS8AO4_WAIT_GSC_AUTOCAL_DONE;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _autocal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATE DONE IRQ TIMED OUT AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually wait for completion in case something terminates our wait early.
	tmp	= gsc_poll_u32(dev, ms, va, AUTOCAL_START, 0);

	if (tmp)
	{
		ms	*= 2;
		ret	= ret ? ret : tmp;
		printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & AUTOCAL_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Final results.
	reg	= os_reg_mem_rx_u32(dev, va);

	if ((reg & AUTOCAL_PASS) == 0)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
				dev->model,
				ms);
	}

	if (ret == 0)
	{
		// Wait for settling.
		os_time_sleep_ms(100);
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_status(dev_data_t* dev, s32* arg)
{
	u32	reg;

	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (reg & AUTOCAL_START)
		arg[0]	= AISS8AO4_AUTOCAL_STATUS_ACTIVE;
	else if ((reg & AUTOCAL_PASS) == 0)
		arg[0]	= AISS8AO4_AUTOCAL_STATUS_FAIL;
	else
		arg[0]	= AISS8AO4_AUTOCAL_STATUS_PASS;

	return(0);
}



//*****************************************************************************
static int _autocal_ai_start(dev_data_t* dev, void* arg)
{
	// Initiate autocalibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, AUTOCAL_AI_START, AUTOCAL_AI_START);
	return(0);
}



//*****************************************************************************
static int _autocal_ai(dev_data_t* dev, void* arg)
{
	u32			bcr;
	long		ms		= dev->cache.autocal_ms + 5000;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	gsc_wait_t	wt;

	if (dev->cache.autocal_ai == 0)
	{
		ret	= -EIO;
	}
	else
	{
		// Safely enable the Autocal Done interrupt.
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_psr_32, IRQ_AUTOCAL_DONE, IRQ_AUTOCAL_DONE);

		// Wait for the local interrupt.
		os_sem_create(&sem);	// dummy, required for wait operations.
		memset(&wt, 0, sizeof(wt));
		wt.flags		= GSC_WAIT_FLAG_INTERNAL;
		wt.gsc			= AISS8AO4_WAIT_GSC_AUTOCAL_DONE;
		wt.timeout_ms	= ms;
		ret				= gsc_wait_event(dev, &wt, _autocal_ai_start, NULL, &sem);
		os_sem_destroy(&sem);

		if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: AI ONLY AUTOCALIBRATE IRQ TIMED OUT AFTER %ld ms.\n",
					dev->model,
					(long) ms);
		}

		// Manually wait for completion in case something terminates our wait early.
		tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, AUTOCAL_AI_START, 0);

		if (tmp)
		{
			ms	*= 2;
			ret	= ret ? ret : tmp;
			printf(	"%s: AI ONLY AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
					dev->model,
					ms);
		}

		// Manually check for completion as the IRQ wait may have ended early.
		bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

		if (bcr & AUTOCAL_START)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: AI ONLY AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
					dev->model,
					ms);
		}

		// Final results.
		bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

		if ((bcr & AUTOCAL_PASS) == 0)
		{
			ret	= ret ? ret : -EIO;
			printf(	"%s: AI ONLY AUTOCALIBRATION FAILED (%ld ms).\n",
					dev->model,
					ms);
		}

		if (ret == 0)
		{
			// Wait for settling.
			os_time_sleep_ms(100);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _ai_buf_clear(dev_data_t* dev, s32* arg)
{
	#define	CLEAR	D13		// BCR
	#define	OVER	D15		// BCR
	#define	UNDER	D23		// BCR

	int		i;
	u32		reg;
	int		ret	= 0;
	VADDR_T	va	= dev->vaddr.gsc_bcr_32;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, va, CLEAR, CLEAR);

	// Wait for the bit to clear.

	for (i = 0;; i++)
	{
		reg	= os_reg_mem_rx_u32(dev, va);

		if ((reg & CLEAR) == 0)
			break;

		if (i >= 250)
		{
			ret	= -EINVAL;
			printf(	"%s: The analog input buffer took too long to clear.\n",
					DEV_NAME);
			break;
		}
	}

	// Clear the Overflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, OVER);

	// Clear the Underflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, UNDER);

	return(ret);
}



//*****************************************************************************
static int _ai_buf_level(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_ibsr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_overflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BUF_ERROR_NO,
		AISS8AO4_BUF_ERROR_YES,
		-1	// terminate list
	};

	int	ret;
	int	ret1;
	int	ret2;

	ret1	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 15, 15);
	arg[0]	= -1;
	ret2	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 15, 15);
	ret		= ret1 ? ret1 : ret2;
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_lvl(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_ibtr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BUF_THR_STS_CLEAR,
		AISS8AO4_BUF_THR_STS_SET,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 14, 14);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_underflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BUF_ERROR_NO,
		AISS8AO4_BUF_ERROR_YES,
		-1	// terminate list
	};

	int	ret;
	int	ret1;
	int	ret2;

	ret1	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 23, 23);
	arg[0]	= -1;
	ret2	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 23, 23);
	ret		= ret1 ? ret1 : ret2;
	return(ret);
}



//*****************************************************************************
static int _ai_burst_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BURST_ENABLE_NO,
		AISS8AO4_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 9, 9);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_size(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_icr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_status(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AI_BURST_STATUS_BUSY,
		AISS8AO4_AI_BURST_STATUS_READY,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _ai_chan_sel(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.ai_chan_qty == 8)
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xFF, dev->vaddr.gsc_icr_32, 31, 24);
	else
		ret	= gsc_s32_range_reg(dev, arg, 0, 0x0F, dev->vaddr.gsc_icr_32, 27, 24);

	return(ret);
}



//*****************************************************************************
static int _ai_clock_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AI_CLOCK_SRC_EXT,
		AISS8AO4_AI_CLOCK_SRC_RAG,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 24, 24);
	return(ret);
}



//*****************************************************************************
static int _ai_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AI_ENABLE_NO,
		AISS8AO4_AI_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 12, 12);
	return(ret);
}



//*****************************************************************************
static int _ai_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options_all[]	=
	{
		AISS8AO4_AI_MODE_DIFF,
		AISS8AO4_AI_MODE_SINGLE,
		AISS8AO4_AI_MODE_ZERO,
		AISS8AO4_AI_MODE_VREF,
		AISS8AO4_AI_MODE_AO_0,
		AISS8AO4_AI_MODE_AO_1,
		AISS8AO4_AI_MODE_AO_2,
		AISS8AO4_AI_MODE_AO_3,
		-1	// terminate list
	};

	static const s32	options_part[]	=
	{
		AISS8AO4_AI_MODE_DIFF,
		AISS8AO4_AI_MODE_SINGLE,
		AISS8AO4_AI_MODE_ZERO,
		AISS8AO4_AI_MODE_VREF,
		-1	// terminate list
	};

	const s32*	ptr	= dev->cache.ao_chan_qty ? options_all : options_part;
	int			ret;

	ret	= gsc_s32_list_reg(dev, arg, ptr, dev->vaddr.gsc_bcr_32, 2, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_range_a(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_RANGE_2_5V,
		AISS8AO4_RANGE_5V,
		AISS8AO4_RANGE_10V,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 5, 4);
	return(ret);
}



//*****************************************************************************
static int _ai_range_b(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_RANGE_2_5V,
		AISS8AO4_RANGE_5V,
		AISS8AO4_RANGE_10V,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 7, 6);
	return(ret);
}



//*****************************************************************************
static int _ai_sw_clock(dev_data_t* dev, s32* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D8, D8);
	return(0);
}



//*****************************************************************************
static int _ai_sw_trigger(dev_data_t* dev, s32* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D11, D11);
	return(0);
}



//*****************************************************************************
static int _ao_access_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_ACCESS_MODE_REG,
		AISS8AO4_AO_ACCESS_MODE_FIFO,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 19, 19);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_buf_clear(dev_data_t* dev, s32* arg)
{
	#undef	CLEAR
	#define	CLEAR	D11		// BOOR
	#define	DATA	D16		// BOOR
	#define	FRAME	D17		// BOOR

	int		i;
	u32		reg;
	int		ret	= 0;
	VADDR_T	va	= dev->vaddr.gsc_boor_32;

	if (dev->cache.ao_chan_qty)
	{
		// Clear the buffer.
		os_reg_mem_mx_u32(dev, va, CLEAR, CLEAR);

		// Wait for the bit to clear.

		for (i = 0;; i++)
		{
			reg	= os_reg_mem_rx_u32(dev, va);

			if ((reg & CLEAR) == 0)
				break;

			if (i >= 250)
			{
				ret	= -EINVAL;
				printf(	"%s: The analog input buffer took too long to clear.\n",
						DEV_NAME);
				break;
			}
		}

		// Clear the Data Overflow status bit.
		os_reg_mem_mx_u32(dev, va, 0, DATA);

		// Clear the Frame Overflow status bit.
		os_reg_mem_mx_u32(dev, va, 0, FRAME);
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_data_over(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BUF_ERROR_NO,
		AISS8AO4_BUF_ERROR_YES,
		-1	// terminate list
	};

	int	ret;
	int	ret1;
	int	ret2;

	if (dev->cache.ao_chan_qty)
	{
		ret1	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 16, 16);
		arg[0]	= -1;
		ret2	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 16, 16);
		ret		= ret1 ? ret1 : ret2;
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_frame_over(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BUF_ERROR_NO,
		AISS8AO4_BUF_ERROR_YES,
		-1	// terminate list
	};

	int	ret;
	int	ret1;
	int	ret2;

	if (dev->cache.ao_chan_qty)
	{
		ret1	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 17, 17);
		arg[0]	= -1;
		ret2	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 17, 17);
		ret		= ret1 ? ret1 : ret2;
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_level(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.ao_chan_qty)
	{
		arg[0]	= -1;
		ret		= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_obsr_32, 18, 0);
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_BUF_MODE_OPEN,
		AISS8AO4_AO_BUF_MODE_CLOSED,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 8, 8);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_buf_thr_lvl(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_obtr_32, 18, 0);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_buf_thr_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BUF_THR_STS_CLEAR,
		AISS8AO4_BUF_THR_STS_SET,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
	{
		arg[0]	= -1;
		ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_obtr_32, 19, 19);
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_status(dev_data_t* dev, s32* arg)
{
	int	ret;
	u32	reg;

	if (dev->cache.ao_chan_qty)
	{
		ret	= 0;
		reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_boor_32);

		if (reg & D12)
			arg[0]	= AISS8AO4_AO_BUF_STATUS_EMPTY;
		else if (reg & D15)
			arg[0]	= AISS8AO4_AO_BUF_STATUS_FULL;
		else if (reg & D13)
			arg[0]	= AISS8AO4_AO_BUF_STATUS_HIGH;
		else
			arg[0]	= AISS8AO4_AO_BUF_STATUS_LOW;
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_burst_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BURST_ENABLE_NO,
		AISS8AO4_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 19, 19);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_burst_status(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_BURST_STATUS_BUSY,
		AISS8AO4_AO_BURST_STATUS_READY,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
	{
		arg[0]	= -1;
		ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 18, 18);
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_chan_sel(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_boor_32, 3, 0);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_clock_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_CLOCK_ENABLE_NO,
		AISS8AO4_AO_CLOCK_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 5, 5);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_clock_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_CLOCK_READY_NO,
		AISS8AO4_AO_CLOCK_READY_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
	{
		arg[0]	= -1;
		ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 6, 6);
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_clock_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_CLOCK_SRC_EXT,
		AISS8AO4_AO_CLOCK_SRC_RCG,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 4, 4);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_load_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_LOAD_READY_NO,
		AISS8AO4_AO_LOAD_READY_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
	{
		arg[0]	= -1;
		ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 10, 10);
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_load_request(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	if (dev->cache.ao_chan_qty)
	{
		// Make the request.
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, D9, D9);

		// Wait for the request to be granted.

		if (dev->io.tx.timeout_s)
		{
			ret	= gsc_poll_u32(dev, dev->io.tx.timeout_s * 1000, dev->vaddr.gsc_boor_32, D9, D9);
		}
	}
	else
	{
		ret	= -EIO;
	}

	return(ret);
}



//*****************************************************************************
static int _ao_range(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_RANGE_2_5V,
		AISS8AO4_RANGE_5V,
		AISS8AO4_RANGE_10V,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 17, 16);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_sw_clock(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	if (dev->cache.ao_chan_qty)
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D20, D20);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_sw_trigger(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	if (dev->cache.ao_chan_qty)
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, D20, D20);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _ao_timing(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_AO_TIMING_IMMED,
		AISS8AO4_AO_TIMING_SIMUL,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 18, 18);
	else
		ret	= -EIO;

	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_DATA_FORMAT_2S_COMP,
		AISS8AO4_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 25, 25);
	return(ret);
}



//*****************************************************************************
static int _dio_dir_out(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;
	u32	v;

	if (arg[0] == -1)
	{
		v		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_diopr_32);
		arg[0]	= ((v & D24) ? 0x2 : 0)
				| ((v & D8 ) ? 0x1 : 0);
	}
	else if ((arg[0] >= 0) && (arg[0] <= 3))
	{
		v		= ((arg[0] & 0x2) ? D24 : 0)
				| ((arg[0] & 0x1) ? D8  : 0);
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_diopr_32, v, D24 | D8);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _dio_read(dev_data_t* dev, s32* arg)
{
	u32	v;

	v		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_diopr_32);
	arg[0]	= ((v & 0x00FF0000) >> 8) | (v & 0x000000FF);
	return(0);
}



//*****************************************************************************
static int _dio_write(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;
	u32	v;

	if (arg[0] == -1)
	{
		ret	= _dio_read(dev, arg);
	}
	else if ((arg[0] >= 0) && (arg[0] <= 0xFFFF))
	{
		v	= (arg[0] << 8) | arg[0];
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_diopr_32, v, 0x00FF00FF);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _gen_a_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_GEN_ENABLE_NO,
		AISS8AO4_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 26, 26);
	return(ret);
}



//*****************************************************************************
static int _gen_a_ndiv(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			dev->cache.ndiv_min_ai,
			dev->cache.ndiv_max_ai,
			dev->vaddr.gsc_ragr_32,
			23,
			0);
	return(ret);
}



//*****************************************************************************
static int _gen_b_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_GEN_ENABLE_NO,
		AISS8AO4_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 27, 27);
	return(ret);
}



//*****************************************************************************
static int _gen_b_ndiv(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			dev->cache.ndiv_min_ai,
			dev->cache.ndiv_max_ai,
			dev->vaddr.gsc_rbgr_32,
			23,
			0);
	return(ret);
}



//*****************************************************************************
static int _gen_c_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_GEN_ENABLE_NO,
		AISS8AO4_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 22, 22);
	return(ret);
}



//*****************************************************************************
static int _gen_c_ndiv(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			dev->cache.ndiv_min_ao,
			dev->cache.ndiv_max_ao,
			dev->vaddr.gsc_rcgr_32,
			23,
			0);
	return(ret);
}



//*****************************************************************************
static int _irq_enable(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFF, dev->vaddr.gsc_psr_32, 14, 0);
	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev, &dev->io.rx);
	return(0);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_overflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_IO_ERROR_IGNORE,
		AISS8AO4_IO_ERROR_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.overflow_check);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AISS8AO4_IO_TIMEOUT_MIN,
			AISS8AO4_IO_TIMEOUT_INFINITE,
			&dev->io.rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _rx_io_underflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_IO_ERROR_IGNORE,
		AISS8AO4_IO_ERROR_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.underflow_check);
	return(ret);
}



//*****************************************************************************
static int _trigger_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_TRIGGER_SRC_EXT,
		AISS8AO4_TRIGGER_SRC_RBG,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 21, 21);
	return(ret);
}



//*****************************************************************************
static int _tx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_write_abort_active_xfer(dev, &dev->io.tx);
	return(0);
}



//*****************************************************************************
static int _tx_io_data_over(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			arg[0]	= dev->io.rx.data_over_check;
			break;

		case AISS8AO4_IO_ERROR_IGNORE:
		case AISS8AO4_IO_ERROR_CHECK:

			dev->io.rx.data_over_check	= arg[0];
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _tx_io_frame_over(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			arg[0]	= dev->io.rx.frame_over_check;
			break;

		case AISS8AO4_IO_ERROR_IGNORE:
		case AISS8AO4_IO_ERROR_CHECK:

			dev->io.rx.frame_over_check	= arg[0];
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _tx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AISS8AO4_IO_TIMEOUT_MIN,
			AISS8AO4_IO_TIMEOUT_INFINITE,
			&dev->io.tx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _burst_sync(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AISS8AO4_BURST_SYNC_DISABLE,
		AISS8AO4_BURST_SYNC_ENABLE,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.burst_sync)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 21, 21);
	}
	else
	{
		arg[0]	= -1;
		ret		= 0;
	}

	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ AISS8AO4_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ AISS8AO4_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ AISS8AO4_IOCTL_REG_MOD,			(void*) gsc_reg_mod_ioctl		},
	{ AISS8AO4_IOCTL_QUERY,				(void*) _query					},
	{ AISS8AO4_IOCTL_INITIALIZE,		(void*) initialize_ioctl		},
	{ AISS8AO4_IOCTL_AUTOCAL,			(void*) _autocal				},
	{ AISS8AO4_IOCTL_AUTOCAL_STATUS,	(void*) _autocal_status			},
	{ AISS8AO4_IOCTL_AUTOCAL_AI,		(void*) _autocal_ai				},
	{ AISS8AO4_IOCTL_AI_BUF_CLEAR,		(void*) _ai_buf_clear			},
	{ AISS8AO4_IOCTL_AI_BUF_LEVEL,		(void*) _ai_buf_level			},
	{ AISS8AO4_IOCTL_AI_BUF_OVERFLOW,	(void*) _ai_buf_overflow		},
	{ AISS8AO4_IOCTL_AI_BUF_THR_LVL,	(void*) _ai_buf_thr_lvl			},
	{ AISS8AO4_IOCTL_AI_BUF_THR_STS,	(void*) _ai_buf_thr_sts			},
	{ AISS8AO4_IOCTL_AI_BUF_UNDERFLOW,	(void*) _ai_buf_underflow		},
	{ AISS8AO4_IOCTL_AI_BURST_ENABLE,	(void*) _ai_burst_enable		},
	{ AISS8AO4_IOCTL_AI_BURST_SIZE,		(void*) _ai_burst_size			},
	{ AISS8AO4_IOCTL_AI_BURST_STATUS,	(void*) _ai_burst_status		},
	{ AISS8AO4_IOCTL_AI_CHAN_SEL,		(void*) _ai_chan_sel			},
	{ AISS8AO4_IOCTL_AI_CLOCK_SRC,		(void*) _ai_clock_src			},
	{ AISS8AO4_IOCTL_AI_ENABLE,			(void*) _ai_enable				},
	{ AISS8AO4_IOCTL_AI_MODE,			(void*) _ai_mode				},
	{ AISS8AO4_IOCTL_AI_RANGE_A,		(void*) _ai_range_a				},
	{ AISS8AO4_IOCTL_AI_RANGE_B,		(void*) _ai_range_b				},
	{ AISS8AO4_IOCTL_AI_SW_CLOCK,		(void*) _ai_sw_clock			},
	{ AISS8AO4_IOCTL_AI_SW_TRIGGER,		(void*) _ai_sw_trigger			},
	{ AISS8AO4_IOCTL_AO_ACCESS_MODE,	(void*) _ao_access_mode			},
	{ AISS8AO4_IOCTL_AO_BUF_CLEAR,		(void*) _ao_buf_clear			},
	{ AISS8AO4_IOCTL_AO_BUF_DATA_OVER,	(void*) _ao_buf_data_over		},
	{ AISS8AO4_IOCTL_AO_BUF_FRAME_OVER,	(void*) _ao_buf_frame_over		},
	{ AISS8AO4_IOCTL_AO_BUF_LEVEL,		(void*) _ao_buf_level			},
	{ AISS8AO4_IOCTL_AO_BUF_MODE,		(void*) _ao_buf_mode			},
	{ AISS8AO4_IOCTL_AO_BUF_THR_LVL,	(void*) _ao_buf_thr_lvl			},
	{ AISS8AO4_IOCTL_AO_BUF_THR_STS,	(void*) _ao_buf_thr_sts			},
	{ AISS8AO4_IOCTL_AO_BUF_STATUS,		(void*) _ao_buf_status			},
	{ AISS8AO4_IOCTL_AO_BURST_ENABLE,	(void*) _ao_burst_enable		},
	{ AISS8AO4_IOCTL_AO_BURST_STATUS,	(void*) _ao_burst_status		},
	{ AISS8AO4_IOCTL_AO_CHAN_SEL,		(void*) _ao_chan_sel			},
	{ AISS8AO4_IOCTL_AO_CLOCK_ENABLE,	(void*) _ao_clock_enable		},
	{ AISS8AO4_IOCTL_AO_CLOCK_READY,	(void*) _ao_clock_ready			},
	{ AISS8AO4_IOCTL_AO_CLOCK_SRC,		(void*) _ao_clock_src			},
	{ AISS8AO4_IOCTL_AO_LOAD_READY,		(void*) _ao_load_ready			},
	{ AISS8AO4_IOCTL_AO_LOAD_REQUEST,	(void*) _ao_load_request		},
	{ AISS8AO4_IOCTL_AO_RANGE,			(void*) _ao_range				},
	{ AISS8AO4_IOCTL_AO_SW_CLOCK,		(void*) _ao_sw_clock			},
	{ AISS8AO4_IOCTL_AO_SW_TRIGGER,		(void*) _ao_sw_trigger			},
	{ AISS8AO4_IOCTL_AO_TIMING,			(void*) _ao_timing				},
	{ AISS8AO4_IOCTL_DATA_FORMAT,		(void*) _data_format			},
	{ AISS8AO4_IOCTL_DIO_DIR_OUT,		(void*) _dio_dir_out			},
	{ AISS8AO4_IOCTL_DIO_READ,			(void*) _dio_read				},
	{ AISS8AO4_IOCTL_DIO_WRITE,			(void*) _dio_write				},
	{ AISS8AO4_IOCTL_GEN_A_ENABLE,		(void*) _gen_a_enable			},
	{ AISS8AO4_IOCTL_GEN_A_NDIV,		(void*) _gen_a_ndiv				},
	{ AISS8AO4_IOCTL_GEN_B_ENABLE,		(void*) _gen_b_enable			},
	{ AISS8AO4_IOCTL_GEN_B_NDIV,		(void*) _gen_b_ndiv				},
	{ AISS8AO4_IOCTL_GEN_C_ENABLE,		(void*) _gen_c_enable			},
	{ AISS8AO4_IOCTL_GEN_C_NDIV,		(void*) _gen_c_ndiv				},
	{ AISS8AO4_IOCTL_IRQ_ENABLE,		(void*) _irq_enable				},
	{ AISS8AO4_IOCTL_RX_IO_ABORT,		(void*) _rx_io_abort			},
	{ AISS8AO4_IOCTL_RX_IO_MODE,		(void*) _rx_io_mode				},
	{ AISS8AO4_IOCTL_RX_IO_OVERFLOW,	(void*) _rx_io_overflow			},
	{ AISS8AO4_IOCTL_RX_IO_TIMEOUT,		(void*) _rx_io_timeout			},
	{ AISS8AO4_IOCTL_RX_IO_UNDERFLOW,	(void*) _rx_io_underflow		},
	{ AISS8AO4_IOCTL_TRIGGER_SRC,		(void*) _trigger_src			},
	{ AISS8AO4_IOCTL_TX_IO_ABORT,		(void*) _tx_io_abort			},
	{ AISS8AO4_IOCTL_TX_IO_DATA_OVER,	(void*) _tx_io_data_over		},
	{ AISS8AO4_IOCTL_TX_IO_FRAME_OVER,	(void*) _tx_io_frame_over		},
	{ AISS8AO4_IOCTL_TX_IO_MODE,		(void*) _tx_io_mode				},
	{ AISS8AO4_IOCTL_TX_IO_TIMEOUT,		(void*) _tx_io_timeout			},
	{ AISS8AO4_IOCTL_WAIT_EVENT,		(void*) gsc_wait_event_ioctl	},
	{ AISS8AO4_IOCTL_WAIT_CANCEL,		(void*) gsc_wait_cancel_ioctl	},
	{ AISS8AO4_IOCTL_WAIT_STATUS,		(void*) gsc_wait_status_ioctl	},
	{ AISS8AO4_IOCTL_BURST_SYNC,		(void*) _burst_sync				},
	{ -1, NULL }
};


