// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/driver/main.h $
// $Rev: 54019 $
// $Date: 2023-12-12 09:15:32 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "16aiss8ao4.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1
#define	OS_METRICS_RX_LOOPS					4000000
#define	OS_METRICS_MX_LOOPS					4000000

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"16AISS8AO4"	// Upper case form of the below.
#define	DEV_NAME							"16aiss8ao4"	// MUST AGREE WITH AISS8AO4_BASE_NAME

#define	DEV_VERSION							"4.5"			// FOR DEVICE SPECIFIC CODE ONLY!
// 4.5	Updated to support the 6.x series kernel.
//		Updated for Fedora 38.
//		Renamed all Auto_Cal content to Autocal.
//		Renamed all Auto_Cal Sts content to Autocal_Status.
//		Updated comments for consistency.
//		Updated the Initialize IOCTL service for consistency.
//		Updated the Autocal IOCTL service for consistency.
//		Updated Input Buffer Clear IOCTL service for consistency.
//		Updated Output Buffer Clear IOCTL service for consistency.
// 4.4	Added support for the 5.x kernel series.
//		Updated initial file content for consistency.
//		Updated for porting to Windows.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL services.
// 4.3	Changed some argument names for clarity.
// 4.2	Updated per extensive changes to common code.
//		Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
// 4.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Added conditional reporting of device metrics during driver startup.
//		Corrected the IOCTL code for the register write and mod services.
// 4.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
// 3.2	Reduced #include list in driver interface header.
// 3.1	Updated for the 3.x kernel.
//		Corrected a system time tick count roll over bug in ioctl.c.
// 3.0	Overhauled.
// 2.3	Ported to Fedora 15.
//		Ported to use of common gsc_kernel_2_[246].[ch] sources.
//		Remove proc entry only after activating it.
// 2.2	Updated various Auto Cal related strings and macros for consistency.
// 2.1	Modified to support the 2.2 and 2.4 kernels.
// 2.0	Renamed numbers items ...a04... to ...ao4..., which refers to Analog
//		Output. Revamped lots of code. Added a couple IOCTL services, junked
//		a couple others. Implemented 32-bit support under 64-bit OS.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_READ_PIO_WORK_32_BIT
#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AISS8AO4_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AISS8AO4_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AISS8AO4_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					data_over_check;	// Check for data overflows?
	s32					frame_over_check;	// Check for frame overflows?
	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					overflow_check;		// Check for overflows?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check for underflows?

	// Initialized by I/O service at time of use.

	int					abort;				// Also set via IOCTL service.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_sem_t			sem;				// Only one transfer at a time.
	os_mem_t			mem;				// I/O buffer.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;			// Analog Input read
		dev_io_t		tx;			// Analog Output write
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bcr_32;		// 0x00	Board Control Register
		VADDR_T			gsc_diopr_32;	// 0x04	Digital I/O Port Register
		VADDR_T			gsc_ibdr_32;	// 0x18	Input Buffer Data Register
		VADDR_T			gsc_ragr_32;	// 0x1C	Rate-A Generator Register
		VADDR_T			gsc_rbgr_32;	// 0x1C	Rate-B Generator Register
		VADDR_T			gsc_icr_32;		// 0x24	Input Configuration Register
		VADDR_T			gsc_ibsr_32;	// 0x28	Input Buffer Size Register
		VADDR_T			gsc_ibtr_32;	// 0x2C	Input Buffer Threshold Register
		VADDR_T			gsc_psr_32;		// 0x30	Primary Status Register
		VADDR_T			gsc_acr_32;		// 0x34	Assembly Configuration Register
		VADDR_T			gsc_boor_32;	// 0x3C	Buffered Output Operations Register
		VADDR_T			gsc_obtr_32;	// 0x40	Output Buffer Threshold Register
		VADDR_T			gsc_obsr_32;	// 0x44	Output Buffer Size Register
		VADDR_T			gsc_obdr_32;	// 0x48	Output Buffer Data Register
		VADDR_T			gsc_rcgr_32;	// 0x4C	Rate-C Generator Register

	} vaddr;

	struct
	{
		s32				ai_chans_max;		// Maximum input channels supported.
		s32				ai_chan_qty;		// The number of input channels on the device.
		s32				ao_chans_max;		// Maximum output channels supported.
		s32				ao_chan_qty;		// The number of output channels on the device.
		s32				autocal_ai;			// Is the AI Autocal option present?
		s32				autocal_ms;			// Maximum ms for autocalibrate

		s32				burst_sync;			// Is Synchronize AI and AO bursting present?

		s32				dmdma;				// Is DMDMA supported?

		s32				fgen_max_ai;		// AI Rate Generator maximum output rate.
		s32				fgen_max_ao;		// AO Rate Generator maximum output rate.
		s32				fgen_min_ai;		// AI Rate Generator minimum output rate.
		s32				fgen_min_ao;		// AO Rate Generator minimum output rate.
		u32				fifo_size_rx;		// Size of Rx FIFO - not the fill level.
		u32				fifo_size_tx;		// Size of Tx FIFO - not the fill level.
		s32				fsamp_max_ai;		// AI The maximum Fsamp rate per channel.
		s32				fsamp_max_ao;		// AO The maximum Fsamp rate per channel.
		s32				fsamp_min_ai;		// AI The minimum Fsamp rate per channel.
		s32				fsamp_min_ao;		// AO The minimum Fsamp rate per channel.

		u32				gsc_acr_32;			// Assembly Configuration Register

		s32				initialize_ms;		// Maximum ms for initialize

		s32				master_clock;		// Master clock frequency

		s32				ndiv_max_ai;		// AI Minimum Ndiv value.
		s32				ndiv_max_ao;		// AO Minimum Ndiv value.
		s32				ndiv_min_ai;		// AI Maximum Ndiv value.
		s32				ndiv_min_ao;		// AO Maximum Ndiv value.

		s32				rate_gen_qty;		// The number of rate generators on device.
		s32				reg_aux_user;		// Number of Auxiliary User Registers.

		s32				sys_io_cfg;			// System I/O Configuration

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
dev_io_t*	dev_io_write_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);
int			dev_write_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
