// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/samples/savedata/perform.c $
// $Rev: 54015 $
// $Date: 2023-12-12 09:05:08 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M					(1024L * 1024L)
#define	TAG_DECODE(v)		GSC_FIELD_DECODE((v),16,16)
#define	TAG_MASK(v)			((v) & 0x10000)
#define	VALUE_MASK(v)		((v) & 0x0FFFF)
#define	UNUSED_MASK			0xFFFE0000



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	int	errs;

	gsc_label("Input Channels");
	errs	= aiss8ao4_query(args->fd, -1, 0, AISS8AO4_QUERY_CHANNEL_AI_QTY, chans);

	if (errs == 0)
	{
		printf("%ld Channels\n", (long) chans[0]);
	}
	else
	{
		printf("FAIL <---\n");
		chans[0]	= 8;	// We'll assume the maximim.
		gsc_label(NULL);
		printf("We'll assume %d channels are present.\n", (int) chans[0]);
	}

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args)
{
	char	buf[80];
	int		errs	= 0;
	int		get		= sizeof(_buffer);
	int		got;
	int		repeat;

	for (repeat = 0; (errs == 0) && (repeat < args->repeat);)
	{
		repeat++;
		strcpy(buf, "Reading");

		if (args->repeat > 1)
		{
			strcat(buf, " #");
			gsc_label_long_comma_buf(repeat, buf + strlen(buf));
		}

		gsc_label(buf);

		got	= aiss8ao4_read(args->fd, _buffer, get);

		if (got < 0)
		{
			errs	= 1;
			printf("FAIL <---  (read error: %d)\n", got);
			break;
		}

		if (got != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (got %ld bytes, requested %ld)\n",
					(long) got,
					(long) get);
			break;
		}

		printf("PASS  (");
		gsc_label_long_comma(get / 4);
		printf(" samples)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _save_string(FILE* file, const char* str)
{
	int	errs;
	int	i;
	int	len;

	len	= (int) strlen(str);
	i	= fprintf(file, "%s", str);

	if (i == len)
	{
		errs	= 0;
	}
	else
	{
		printf("FAIL <---  (fprintf() failure)\n");
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _save_eol(FILE* file)
{
	int	errs;

	errs	= _save_string(file, "\r\n");
	return(errs);
}



//*****************************************************************************
static int _save_headers(const args_t* args, FILE* file)
{
	int	argc;
	int	errs;

	errs	= _save_string(file, "#");

	for (argc = 0; argc < args->argc; argc++)
	{
		errs	+= _save_string(file, " ");
		errs	+= _save_string(file, args->argv[argc]);
	}

	errs	+= _save_eol(file);
	return(errs);
}



//*****************************************************************************
static int _save_value(const args_t* args, FILE* file, u32 value)
{
	char	buf1[40];
	char	buf2[80];
	int		errs;

	if (args->chan_tag == CHAN_TAG_EXCLUDE)
		value	= VALUE_MASK(value);
	else if (args->chan_tag == CHAN_TAG_ONLY)
		value	= TAG_MASK(value);

	if (args->decimal)
		sprintf(buf1, "%ld", (long) value);
	else
		sprintf(buf1, "%08lX", (long) value);

	sprintf(buf2, "  %8s", buf1);
	errs	=  _save_string(file, buf2);
	return(errs);
}



//*****************************************************************************
static int _validate_data(int chans, u32 last, long index, u32 data)
{
	static	int	chan_tag	= 0;
	int			errs		= 0;
	int			expect;
	static	int	last_tag	= 0;

	if (data & UNUSED_MASK)
	{
		errs	= 1;
		printf(" FAIL <---  (Upper bits are invalid: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	chan_tag	= TAG_DECODE(data) ? 0 : chan_tag + 1;

	if (chan_tag >= chans)
	{
		errs	= 1;
		printf(" FAIL <---  (Invalid Channel Tag: [%ld] = 0x%08lX)\n", index, (long) data);
	}

	if (last == 0xFFFFFFFF)
	{
		if (chan_tag != 0)
		{
			errs	= 1;
			printf(" FAIL <---  (First tag not zero: [%ld] = 0x%08lX)\n", index, (long) data);
		}
	}
	else
	{
		last_tag	= TAG_DECODE(last) ? 0 : last_tag + 1;
		expect		= (last_tag + 1) % chans;

		if (chan_tag != expect)
		{
			errs	= 1;
			printf(	" FAIL <---  (Lost Synchronozation: [%ld] = 0x%08lX, [%ld] = 0x%08lX)\n",
					index - 1,
					(long) last,
					index,
					(long) data);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(const args_t* args, int chans, int errs)
{
	char		buf[64];
	FILE*		file;
	long		index;
	u32			last	= 0xFFFFFFFF;
	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;
	int			tag_got;

	strcpy(buf, "Saving");

	if (args->force_save)
	{
		errs	= 0;
		strcat(buf, " (Forced)");
	}

	gsc_label(buf);

	for (;;)
	{
		if ((errs) && (args->force_save == 0))
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		errs	+= _save_headers(args, file);

		for (index = 0; index < samples; index++)
		{
			tag_got	= TAG_DECODE(_buffer[index]) ? 0 : 1;

			if ((index > 0) && (tag_got == 0))
				errs	+= _save_eol(file);

			errs	+= _save_value(args, file, _buffer[index]);
			errs	+= _validate_data(chans, last, index, _buffer[index]);
			last	= _buffer[index];
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configure");
	printf("\n");
	gsc_label_level_inc();

	errs	+= aiss8ao4_config_ai		(args->fd, -1, 1, args->fsamp);
	errs	+= aiss8ao4_ai_mode			(args->fd, -1, 1, args->ai_mode, NULL);
	errs	+= aiss8ao4_rx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= aiss8ao4_rx_io_overflow	(args->fd, -1, 1, AISS8AO4_IO_ERROR_IGNORE, NULL);
	errs	+= aiss8ao4_rx_io_underflow	(args->fd, -1, 1, AISS8AO4_IO_ERROR_IGNORE, NULL);
	errs	+= aiss8ao4_rx_io_timeout	(args->fd, -1, 1, 30, NULL);
	errs	+= aiss8ao4_ai_buf_underflow(args->fd, -1, 1, AISS8AO4_BUF_ERROR_CLEAR, NULL);
	errs	+= aiss8ao4_ai_buf_overflow	(args->fd, -1, 1, AISS8AO4_BUF_ERROR_CLEAR, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chans	= 16;
	int	errs	= 0;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _config					(args);
	errs	+= _channels				(args, &chans);
	errs	+= aiss8ao4_ai_buf_clear	(args->fd, -1, 1);
	errs	+= _read_data				(args);
	errs	+= aiss8ao4_ai_buf_overflow	(args->fd, -1, 1, -1, NULL);
	errs	+= aiss8ao4_ai_buf_underflow(args->fd, -1, 1, -1, NULL);
	errs	+= _save_data				(args, chans, errs);

	gsc_label_level_dec();
	return(errs);
}



