// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/samples/txrate/perform.c $
// $Rev: 54014 $
// $Date: 2023-12-12 09:04:16 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M	(1024L * 1024L)



// variables ******************************************************************

static u32	_data[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	int	errs	= 0;
	s32	get;
	s32	tmp;

	gsc_label("Output Channels");

	get			= -1;
	errs		+= aiss8ao4_ao_chan_sel(args->fd, -1, 0, -1, &get);
	chans[0]	= 0;

	for (tmp = get; tmp ; tmp >>= 1)
	{
		if (tmp & 0x1L)
			chans[0]++;
	}

	printf(	"%ld Channel%s  (0x%lX)\n",
			(long) chans[0],
			(chans[0] == 1) ? "" : "s",
			(long) get);
	return(errs);
}



//*****************************************************************************
static int _prefill_fifo(const args_t* args)
{
	int	errs	= 0;
	s32	sts;

	gsc_label("Pre-Fill Output FIFO");

	for (; errs == 0;)
	{
		errs	+= aiss8ao4_reg_write(args->fd, -1, 0, AISS8AO4_GSC_OBDR, 0x8000);
		errs	+= aiss8ao4_ao_buf_status(args->fd, -1, 0, &sts);

		if (sts == AISS8AO4_AO_BUF_STATUS_FULL)
			break;
	}

	if (errs == 0)
		printf("PASS  (Output buffer is now full.)\n");

	return(errs);
}



//*****************************************************************************
static int _rate_test(const args_t* args, long send)
{
	os_time_ns_t	begin;
	os_time_ns_t	end;
	int				errs		= 0;
	long long		data_limit	= args->tx_mb * 1000000;
	long long		data_total	= 0;
	s32				fifo;
	long long		ms;
	long long		ms_limit	= 1000 * args->seconds;
	long long		ms_total	= 0;
	double			rate;
	long			sent;
	s32				timeout;
	long long		total;

	for (;;)	// A convenience loop.
	{
		memset(_data, 0x01, sizeof(_data));

		// Prefill the output buffer
		gsc_label("Prefill Buffer");
		errs	+= aiss8ao4_tx_io_timeout(args->fd, -1, 0, -1, &timeout);
		errs	+= aiss8ao4_tx_io_timeout(args->fd, -1, 0, 0, NULL);
		errs	+= aiss8ao4_query(args->fd, -1, 0, AISS8AO4_QUERY_FIFO_SIZE_TX, &fifo);
		fifo	*= 4;
		send	= sizeof(_data);
		total	= 0;

		for (;;)
		{
			sent	= aiss8ao4_write(args->fd, _data, send);

			if (sent < 0)
				errs++;

			if (errs)
				break;

			if (sent < (send / 10))
				break;

			total	+= sent;

			if (total >= (fifo * 2))	// make extra large, fsamp may be very high
				break;
		}

		errs	+= aiss8ao4_tx_io_timeout(args->fd, -1, 0, timeout, NULL);
		printf("%s\n", errs ? "FAIL <---" : "PASS");

		if (errs)
			break;

		// Write the data to the device.
		gsc_label("Writing Data");
		os_time_get_ns(&begin);

		for (;;)
		{
			sent	= aiss8ao4_write(args->fd, _data, send);

			if (sent < 0)
			{
				errs++;
				printf("FAIL <--- (aiss8ao4_write, errno %d)\n", errno);
				break;
			}

			os_time_get_ns(&end);
			data_total	+= sent;
			ms_total	= ((long long) end.tv_sec * 1000 + end.tv_nsec / 1000000)
						- ((long long) begin.tv_sec * 1000 + begin.tv_nsec / 1000000);

			if ((ms_total >= ms_limit) && (data_total >= data_limit))
				break;
		}

		printf(	"%s  (", errs ? "FAIL <---" : "PASS");

		// Volume of data
		total	= data_total;
		gsc_label_long_comma(total / 4);
		printf(" Samples, ");

		// Period
		ms	= ms_total;
		gsc_label_float_comma((double) ms / 1000, -1, 3);
		printf(" Seconds, ");

		// Rate
		ms		= (ms <= 0) ? 1 : ms;
		rate	= ((double) total * 1000000 / 4) / ms / 1000;
		gsc_label_float_comma(rate, -1, 3);
		printf(" S/S)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32		chans	= 4;
	int		errs	= 0;
	s32		fsamp	= 0;
	long	send	= sizeof(_data);

	errs	+= aiss8ao4_query			(args->fd, -1, 1, AISS8AO4_QUERY_FSAMP_MAX_AO, &fsamp);
	errs	+= aiss8ao4_config_ao		(args->fd, -1, 1, fsamp);
	errs	+= aiss8ao4_tx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= _channels				(args, &chans);
	errs	+= aiss8ao4_tx_io_data_over	(args->fd, -1, 1, AISS8AO4_IO_ERROR_IGNORE, NULL);
	errs	+= aiss8ao4_tx_io_frame_over(args->fd, -1, 1, AISS8AO4_IO_ERROR_IGNORE, NULL);
	errs	+= aiss8ao4_ao_clock_enable	(args->fd, -1, 1, AISS8AO4_AO_CLOCK_ENABLE_NO, NULL);
	errs	+= _prefill_fifo			(args);
	errs	+= aiss8ao4_ao_clock_enable	(args->fd, -1, 1, AISS8AO4_AO_CLOCK_ENABLE_YES, NULL);

	gsc_label ("Write Rate");
	printf("(%d second minimum, %ld MB minimum)\n", args->seconds, (long) args->tx_mb);
	gsc_label_level_inc();
	errs	+= _rate_test(args, send);
	gsc_label_level_dec();
	errs	+= aiss8ao4_ao_buf_data_over(args->fd, -1, 1, -1, NULL);
	return(errs);
}



