// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/utils/util_ai_mode.c $
// $Rev: 43450 $
// $Date: 2018-08-29 15:56:40 -0500 (Wed, 29 Aug 2018) $

// 16AISS8AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao4_ai_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO4_IOCTL_AI_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aiss8ao4_ai_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("AI Mode", index);

	ret		= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_AI_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AISS8AO4_AI_MODE_DIFF:

			ptr	= "Differential";
			break;

		case AISS8AO4_AI_MODE_SINGLE:

			ptr	= "Single Ended";
			break;

		case AISS8AO4_AI_MODE_ZERO:

			ptr	= "Zero Volt Test";
			break;

		case AISS8AO4_AI_MODE_VREF:

			ptr	= "Vref Test";
			break;

		case AISS8AO4_AI_MODE_AO_0:

			ptr	= "AO Channel 0 Loopback";
			break;

		case AISS8AO4_AI_MODE_AO_1:

			ptr	= "AO Channel 1 Loopback";
			break;

		case AISS8AO4_AI_MODE_AO_2:

			ptr	= "AO Channel 2 Loopback";
			break;

		case AISS8AO4_AI_MODE_AO_3:

			ptr	= "AO Channel 3 Loopback";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


