// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/utils/util_burst_sync.c $
// $Rev: 51760 $
// $Date: 2022-10-12 10:25:01 -0500 (Wed, 12 Oct 2022) $

// 16AISS8AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao4_burst_sync
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO4_IOCTL_BURST_SYNC service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aiss8ao4_burst_sync(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("AI Burst Status", index);

	ret		= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_BURST_SYNC, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case AISS8AO4_BURST_SYNC_DISABLE:

			ptr	= "Disabled";
			break;

		case AISS8AO4_BURST_SYNC_ENABLE:

			ptr	= "Enabled";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


