// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/utils/util_config_ai.c $
// $Rev: 54027 $
// $Date: 2024-03-01 15:52:34 -0600 (Fri, 01 Mar 2024) $

// 16AISS8AO4: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_200K	(200L * 1024L)



/******************************************************************************
*
*	Function:	aiss8ao4_config_ai
*
*	Purpose:
*
*		Configure the given device's analog input options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao4_config_ai(int fd, int index, int verbose, s32 fsamp)
{
	s32	chans;
	int	errs	= 0;
	s32	mask;
	s32	ndiv;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aiss8ao4_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= aiss8ao4_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= aiss8ao4_rx_io_overflow		(fd, index, verbose, AISS8AO4_IO_ERROR_CHECK,		NULL);
	errs	+= aiss8ao4_rx_io_timeout		(fd, index, verbose, 10,							NULL);
	errs	+= aiss8ao4_rx_io_underflow		(fd, index, verbose, AISS8AO4_IO_ERROR_CHECK,		NULL);

	errs	+= aiss8ao4_data_format			(fd, index, verbose, AISS8AO4_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= aiss8ao4_ai_range_a			(fd, index, verbose, AISS8AO4_RANGE_10V,			NULL);
	errs	+= aiss8ao4_ai_range_b			(fd, index, verbose, AISS8AO4_RANGE_10V,			NULL);

	errs	+= aiss8ao4_query				(fd, index, verbose, AISS8AO4_QUERY_CHANNEL_AI_QTY, &chans);
	mask	= (chans == 4) ? 0xF : 0xFF;
	errs	+= aiss8ao4_ai_chan_sel			(fd, index, verbose, mask,							NULL);

	errs	+= aiss8ao4_ai_mode				(fd, index, verbose, AISS8AO4_AI_MODE_DIFF,			NULL);
	errs	+= aiss8ao4_ai_buf_thr_lvl		(fd, index, verbose, _200K,							NULL);
	errs	+= aiss8ao4_ai_burst_size		(fd, index, verbose, 0,								NULL);
	errs	+= aiss8ao4_ai_burst_enable		(fd, index, verbose, AISS8AO4_BURST_ENABLE_NO,		NULL);
	errs	+= aiss8ao4_fsamp_ai_compute	(fd, index, verbose, fsamp, &ndiv,					NULL);
	errs	+= aiss8ao4_gen_a_ndiv			(fd, index, verbose, ndiv,							NULL);
	errs	+= aiss8ao4_gen_a_enable		(fd, index, verbose, AISS8AO4_GEN_ENABLE_YES,		NULL);
	errs	+= aiss8ao4_gen_b_enable		(fd, index, verbose, AISS8AO4_GEN_ENABLE_NO,		NULL);
	errs	+= aiss8ao4_ai_clock_src		(fd, index, verbose, AISS8AO4_AI_CLOCK_SRC_RAG,		NULL);
	errs	+= aiss8ao4_trigger_src			(fd, index, verbose, AISS8AO4_TRIGGER_SRC_RBG,		NULL);
	errs	+= aiss8ao4_ai_enable			(fd, index, verbose, AISS8AO4_AI_ENABLE_YES,		NULL);

	errs	+= aiss8ao4_fsamp_ai_report_all	(fd, index, verbose,								NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the board user manual for clarification.
	errs	+= aiss8ao4_autocal				(fd, index, verbose);	// Cable may need disconnecting for this.

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= aiss8ao4_irq_enable			(fd, index, verbose, 0,								NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the overflow and underflow status bits.
	errs	+= aiss8ao4_ai_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


