// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/utils/util_config_ao.c $
// $Rev: 54006 $
// $Date: 2023-12-12 08:58:57 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_200K	(200L * 1024L)



/******************************************************************************
*
*	Function:	aiss8ao4_config_ao
*
*	Purpose:
*
*		Configure the given device's analog output options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao4_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	s32	chans;
	int	errs	= 0;
	s32	mask;
	s32	ndiv;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= aiss8ao4_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= aiss8ao4_tx_io_data_over		(fd, index, verbose, AISS8AO4_IO_ERROR_CHECK,		NULL);
	errs	+= aiss8ao4_tx_io_frame_over	(fd, index, verbose, AISS8AO4_IO_ERROR_CHECK,		NULL);
	errs	+= aiss8ao4_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= aiss8ao4_tx_io_timeout		(fd, index, verbose, 10,							NULL);

	errs	+= aiss8ao4_data_format			(fd, index, verbose, AISS8AO4_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= aiss8ao4_ao_range			(fd, index, verbose, AISS8AO4_RANGE_10V,			NULL);

	errs	+= aiss8ao4_query				(fd, index, verbose, AISS8AO4_QUERY_CHANNEL_AO_QTY, &chans);
	mask	= chans ? 0xF : 0;
	errs	+= aiss8ao4_ao_chan_sel			(fd, index, verbose, mask,							NULL);

	errs	+= aiss8ao4_ao_access_mode		(fd, index, verbose, AISS8AO4_AO_ACCESS_MODE_FIFO,	NULL);
	errs	+= aiss8ao4_ao_buf_mode			(fd, index, verbose, AISS8AO4_AO_BUF_MODE_OPEN,		NULL);
	errs	+= aiss8ao4_ao_timing			(fd, index, verbose, AISS8AO4_AO_TIMING_SIMUL,		NULL);
	errs	+= aiss8ao4_ao_buf_thr_lvl		(fd, index, verbose, _200K,							NULL);
	errs	+= aiss8ao4_ao_burst_enable		(fd, index, verbose, AISS8AO4_BURST_ENABLE_NO,		NULL);
	errs	+= aiss8ao4_fsamp_ao_compute	(fd, index, verbose, fsamp, &ndiv,					NULL);
	errs	+= aiss8ao4_gen_c_ndiv			(fd, index, verbose, ndiv,							NULL);
	errs	+= aiss8ao4_gen_c_enable		(fd, index, verbose, AISS8AO4_GEN_ENABLE_YES,		NULL);
	errs	+= aiss8ao4_gen_b_enable		(fd, index, verbose, AISS8AO4_GEN_ENABLE_NO,		NULL);
	errs	+= aiss8ao4_ao_clock_src		(fd, index, verbose, AISS8AO4_AO_CLOCK_SRC_RCG,		NULL);
	errs	+= aiss8ao4_trigger_src			(fd, index, verbose, AISS8AO4_TRIGGER_SRC_RBG,		NULL);
	errs	+= aiss8ao4_ao_clock_enable		(fd, index, verbose, AISS8AO4_AO_CLOCK_ENABLE_YES,	NULL);

	errs	+= aiss8ao4_fsamp_ao_report_all	(fd, index, verbose,								NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the board user manual for clarification.
	errs	+= aiss8ao4_autocal				(fd, index, verbose);

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= aiss8ao4_irq_enable			(fd, index, verbose, 0,								NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the data overflow and frame overflow status bits.
	errs	+= aiss8ao4_ao_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


