// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/utils/util_id.c $
// $Rev: 54006 $
// $Date: 2023-12-12 08:58:57 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_id_device_pci
*
*	Purpose:
*
*		Identify the device using the PCI registers.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

static int _id_device_pci(int fd)
{
	int	errs;
	u32	reg;

	gsc_label("Vendor ID");
	errs	= aiss8ao4_reg_read(fd, -1, 0, GSC_PCI_9056_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= aiss8ao4_reg_read(fd, -1, 0, GSC_PCI_9056_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9056)
	{
		printf("(PCI9056)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= aiss8ao4_reg_read(fd, -1, 0, GSC_PCI_9056_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= aiss8ao4_reg_read(fd, -1, 0, GSC_PCI_9056_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x3172)
	{
		printf("(16AISS8AO4)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



//*****************************************************************************
static int _id_device_acr(int fd)
{
	u32	acr;
	int	errs;

	gsc_label("ACR");
	errs	= aiss8ao4_reg_read(fd, -1, 0, AISS8AO4_GSC_ACR, &acr);
	printf("0x%08lX\n", (long) acr);
	return(errs);
}



/******************************************************************************
*
*	Function:	aiss8ao4_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aiss8ao4_id_device(int fd, int index, int verbose)
{
	s32		device_type;
	int		errs;
	char*	psz			= "";

	if (verbose)
		gsc_label_index("Device", index);

	errs	= aiss8ao4_query(fd, -1, 0, AISS8AO4_QUERY_DEVICE_TYPE, &device_type);

	switch (device_type)
	{
		default:

			errs	= 1;

			if (verbose)
			{
				printf(	"FAIL <---  (unexpected device type: %ld)",
						(long) device_type);
			}

			break;

		case GSC_DEV_TYPE_16AISS8AO4:

			psz	= "16AISS8AO4";
			break;
	}

	if (verbose)
	{
		printf("%s\n", psz);
		gsc_label_level_inc();
	}

	errs	+= _id_device_pci(fd);
	errs	+= _id_device_acr(fd);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



