// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AISS8AO4/16aiss8ao4_Linux_3.x.x.x_gsc_dn/utils/util_query.c $
// $Rev: 54006 $
// $Date: 2023-12-12 08:58:57 -0600 (Tue, 12 Dec 2023) $

// 16AISS8AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aiss8ao4_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AISS8AO4_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aiss8ao4_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AISS8AO4_QUERY_AUTOCAL_AI:		ptr	= "AI Autocal Support";		break;
		case AISS8AO4_QUERY_AUTOCAL_MS:		ptr	= "Autocal Duration";		break;
		case AISS8AO4_QUERY_BURST_SYNC:		ptr	= "Burst Sync";				break;
		case AISS8AO4_QUERY_CHANNEL_AI_MAX:	ptr	= "AI Max Channels";		break;
		case AISS8AO4_QUERY_CHANNEL_AI_QTY:	ptr	= "AI Installed Channels";	break;
		case AISS8AO4_QUERY_CHANNEL_AO_MAX:	ptr	= "AO Max Channels";		break;
		case AISS8AO4_QUERY_CHANNEL_AO_QTY:	ptr	= "AO Installed Channels";	break;
		case AISS8AO4_QUERY_COUNT:			ptr	= "Query Options";			break;
		case AISS8AO4_QUERY_DEVICE_TYPE:	ptr	= "Device Type";			break;
		case AISS8AO4_QUERY_DMDMA:			ptr	= "DMDMA Supported";		break;
		case AISS8AO4_QUERY_FGEN_MAX_AI:	ptr	= "AI Fgen Max";			break;
		case AISS8AO4_QUERY_FGEN_MIN_AI:	ptr	= "AI Fgen Min";			break;
		case AISS8AO4_QUERY_FGEN_MAX_AO:	ptr	= "AO Fgen Max";			break;
		case AISS8AO4_QUERY_FGEN_MIN_AO:	ptr	= "AO Fgen Min";			break;
		case AISS8AO4_QUERY_FIFO_SIZE_RX:	ptr	= "Rx FIFO Size";			break;
		case AISS8AO4_QUERY_FIFO_SIZE_TX:	ptr	= "Tx FIFO Size";			break;
		case AISS8AO4_QUERY_FSAMP_MAX_AI:	ptr	= "AI Fsamp Max";			break;
		case AISS8AO4_QUERY_FSAMP_MAX_AO:	ptr	= "AO Fsamp Max";			break;
		case AISS8AO4_QUERY_FSAMP_MIN_AI:	ptr	= "AI Fsamp Min";			break;
		case AISS8AO4_QUERY_FSAMP_MIN_AO:	ptr	= "AO Fsamp Min";			break;
		case AISS8AO4_QUERY_INIT_MS:		ptr	= "Init. Duration";			break;
		case AISS8AO4_QUERY_MASTER_CLOCK:	ptr	= "Master Clock";			break;
		case AISS8AO4_QUERY_NDIV_MAX_AI:	ptr	= "AI Ndiv Max";			break;
		case AISS8AO4_QUERY_NDIV_MAX_AO:	ptr	= "AO Ndiv Max";			break;
		case AISS8AO4_QUERY_NDIV_MIN_AI:	ptr	= "AI Ndiv Min";			break;
		case AISS8AO4_QUERY_NDIV_MIN_AO:	ptr	= "AO Ndiv Min";			break;
		case AISS8AO4_QUERY_RATE_GEN_QTY:	ptr	= "Rate Gen Qty";			break;
		case AISS8AO4_QUERY_REG_AUX_USER:	ptr	= "Aux User Regs";			break;
		case AISS8AO4_QUERY_SYS_IO_CFG:		ptr	= "System I/O Config";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= aiss8ao4_ioctl(fd, AISS8AO4_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AISS8AO4_QUERY_CHANNEL_AI_MAX:
		case AISS8AO4_QUERY_CHANNEL_AI_QTY:
		case AISS8AO4_QUERY_CHANNEL_AO_MAX:
		case AISS8AO4_QUERY_CHANNEL_AO_QTY:
		case AISS8AO4_QUERY_COUNT:
		case AISS8AO4_QUERY_NDIV_MAX_AI:
		case AISS8AO4_QUERY_NDIV_MAX_AO:
		case AISS8AO4_QUERY_NDIV_MIN_AI:
		case AISS8AO4_QUERY_NDIV_MIN_AO:
		case AISS8AO4_QUERY_RATE_GEN_QTY:
		case AISS8AO4_QUERY_REG_AUX_USER:

			sprintf(buf, "%ld", (long) set);
			break;

		case AISS8AO4_QUERY_FGEN_MAX_AI:
		case AISS8AO4_QUERY_FGEN_MAX_AO:
		case AISS8AO4_QUERY_FGEN_MIN_AI:
		case AISS8AO4_QUERY_FGEN_MIN_AO:
		case AISS8AO4_QUERY_MASTER_CLOCK:

			sprintf(buf, "%ld Hz", (long) set);
			break;

		case AISS8AO4_QUERY_AUTOCAL_MS:
		case AISS8AO4_QUERY_INIT_MS:

			sprintf(buf, "%ld ms", (long) set);
			break;

		case AISS8AO4_QUERY_FSAMP_MAX_AI:
		case AISS8AO4_QUERY_FSAMP_MAX_AO:
		case AISS8AO4_QUERY_FSAMP_MIN_AI:
		case AISS8AO4_QUERY_FSAMP_MIN_AO:

			sprintf(buf, "%ld S/S", (long) set);
			break;

		case AISS8AO4_QUERY_AUTOCAL_AI:
		case AISS8AO4_QUERY_BURST_SYNC:
		case AISS8AO4_QUERY_DMDMA:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case AISS8AO4_QUERY_FIFO_SIZE_RX:
		case AISS8AO4_QUERY_FIFO_SIZE_TX:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			sprintf(buf, "%ld%s Words", (long) tmp, ptr);
			break;

		case AISS8AO4_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16AISS8AO4)
			{
				strcpy(buf, "16AISS8AO4");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case AISS8AO4_QUERY_SYS_IO_CFG:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

					case AISS8AO4_SYS_IO_CFG_STANDARD:	strcpy(buf, "Standard");	break;
					case AISS8AO4_SYS_IO_CFG_ALT_SMA:	strcpy(buf, "Alt SMA");		break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


